pro plot_ace, pt0, pt1, acedata, _extra=_extra, ytype=ytype, $
  electrons_only=electrons_only, protons_only=protons_only, $
  nowindow=nowindow, nolegend=nolegend, zbuffer=zbuffer, log=log, $
  debug=debug

;+
;   Name: plot_ace
;
;   Purpose: plot ACE timeseries for user selected time/instrument
;
;   Input Parameters:
;      pt0, pt1 - time range (any SSW formats)
;
;   Keyword Parameters:
;      /EPAM, /SWEPAM, /SIS, /MAG - which ACE instrument to plot
;
;      ytype - rsi standard ( /YTYPE <=> YTYPE=1 <=> log plot )
;
;      protons_only - if set, only plot protons (EPAM only)
;      electrons_only - if set, only plot electrons (EPAM only)
;
;   History:
;      19-October-2001 - S.L.Freeland 
;
;   Calls:
;      get_acedata, reltime, usual ssw suspects
;
;-

case n_params() of 
   0: begin 
         t1=reltime(/now, out_style='vms')
         t0=reltime(days=-1, /day_only,out_style='vms')
   endcase
   1: begin 
        t0=reltime(pt0,/day_only,out_style='vms')
        t1=reltime(pt0,days=1,out_style='vms',/day_only)
   endcase
   else: begin 
      t0=anytim(pt0,/vms)
      t1=anytim(pt1,/vms)
   endcase
endcase

daily=ssw_deltat(t0, ref=reltime(/now),/day) gt -31
 
acedata=get_acedata(t0,t1,daily=daily,_extra=_extra)
if not data_chk(acedata,/struct) then begin 
   box_message,'No ACE data for time range: '  + anytim([t0,t1],/vms,/trunc)
   return
endif

tn=tag_names(acedata)
npc=total(strmid(tn,0,1) eq 'P')>0
nec=total(strmid(tn,0,1) eq 'E')>0

if not exist(ytype) then ytype=0
; ---------- set switch states ----------------
proton_only=keyword_set(proton_only)
electron_only=keyword_set(electron_only)
reverse=keyword_set(reverse)
nodefcolors=keyword_set(nodefcolors) or reverse
debug=keyword_set(debug)
nolegend=keyword_set(nolegend)
nowindow=keyword_set(nowindow)
zbuffer=keyword_set(zbuffer)
log=keyword_set(log) or (ytype and '1'x)   ; need this for later...
ytype=ytype or log
; ---------------------------------------------

; ------------- define channel subsets of interest --------------

case 1 of 
   npc eq 0: 
   n_elements(pchannels) eq 0: pchannels=indgen(npc)
   else: pchannels=pchannels>0<(npc-1)
endcase

case 1 of 
   nec eq 0:
   n_elements(echannels) eq 0: echannels=indgen(nec)
   else: echannels=ehannels>0<(nec-1)
endcase


dtemp=!d.name                              ; dont clobber plot device

case 1 of
   nowindow and zbuffer: set_plot,'Z'
   nowindow:
   else: wdef,xx,768,512, zbuffer=zbuffer, title='GOES Particle Events' ; <<<< TODO
endcase

if not keyword_set(nodefcolors) then begin 
   linecolors                                           ; load 'fancy' colors
   goodcolors=[4,5,7,9,12,2]                            ; linecolors subset
   pcolors=(goodcolors)(0:npc-1)                        ; proton colors
   ecolors=(shift(goodcolors,-3))(0:nec-1)              ; electron (shift)
   background=11
   maincolor=!p.color
endif else begin 
   pcolors=replicate(([!p.color,0])(reverse),npc)
   ecolors=replicate(([!p.color,0])(reverse),nec)
   background=([0,!p.color])(reverse)
   maincolor=!p.color-background
endelse


; ----------- scale plot to maximum (P/E) --------------------

case 1 of 
   required_tags(acedata,'P_DENSITY,B_SPEED,ION_TEMP'): begin 
      mtitle='ACE/SWEPAM'
      init='B_SPEED'
      p0max=max(gt_tagval(acedata,init))
      units='Bulk Speed (Km/Hr)'
      plabels='Bulk Speed'
      ptags=init
      etags=init
      proton_only=1
   endcase
   required_tags(acedata, 'E38_53,P112_187'): begin  
      mtitle='ACE/EPAM'
      init='P47_65'
      poma=max(gt_tagval(acedata,init))
      units='Differential Flux/cm2-s-ster-MeV'
      tn=tag_names(acedata)
      ptags=tn(where(strmid(tn,0,1) eq 'P'))
      etags=tn(where(strmid(tn,0,1) eq 'E')) 
      p0=strsplit(ptags,'_',/head,tail=p1)
      e0=strsplit(etags,'_',/head,tail=e1)
      plabels='Prot: ' + strjustify(strtrim(str2number(p0),2)) + $
              ' - ' + strjustify(strtrim(str2number(p1),2))
      elabels='Elec: ' + strjustify(strtrim(str2number(e0),2)) + ' - ' + $
                         strjustify(strtrim(str2number(e1),2))

   endcase
   required_tags(acedata,'P_GT10MEV,P_GT30MEV'): begin 
      mtitle='ACE/SIS Protons'
      units='Proton flux p/cs2-sec-ster'
      proton_only=1
      init='P_GT10MEV'
      plabels=['Protons > 10MeV','Protons > 30 MeV']
      elabels=''
      ptags=['P_GT10MEV','P_GT30MEV']
   endcase
   else: begin
      box_message,'Sorry, EPAM, SWEPAM or SIS only..'
      return
   endcase

endcase

init=init+(['>0','>.01'])(log)                     ; log protect

; ------------- initialize the plot ----------------
ytitle=units
if log and not exist(ytype) then ytype=log

initcmd='utplot, acedata, acedata.'+init + $
  ',ytype=ytype,title=mtitle,ytitle=ytitle,color=maincolor,background=background,/xstyle,/nodata,_extra=_extra'
estat=execute(initcmd)
if electron_only then npc=0
if proton_only   then nec=0

npc=n_elements(pchannels)
nec=n_elements(echannels)

if debug then stop,'npc'
ppsym=1           
epsym=4
for i=0, npc-1 do begin  
  print,ptags(i)
  outplot,acedata,acedata.(tag_index(acedata,ptags(i))), color=pcolors(i), psym=ppsym
endfor

for i=0,nec-1 do $
   outplot,acedata,acedata.(tag_index(acedata,etags(i))),color=ecolors(i), psym=epsym


; ----------------- add a legend ---------------------------------------
if not keyword_set(nolegend) then begin            ; build legend info
   lcols=-1
   llabs=''
   lpsym=-1
   if npc gt 0 then begin                          ; add Proton info
      lcols=[lcols,pcolors(0:npc-1)]
      llabs=[llabs,plabels(0:npc-1)]
      lpsym=[lpsym,replicate(ppsym,npc)]
   endif    
 
   if nec gt 0 then begin                          ; add Electron info
      lcols=[lcols,ecolors(0:nec-1)]
      llabs=[llabs,elabels(0:nec-1)]
      lpsym=[lpsym,replicate(epsym,nec)]   
   endif

   if n_elements(lcols) gt 1 then begin 
      legend,llabs(1:*), psym=lpsym(1:*), colors=lcols(1:*), textcolors=!p.color-background ; add legend
   endif
endif


if debug then stop,'debug...'

set_plot,dtemp                                      ; restore users plot device

return
end
